;
; MilesAhead include script with a few system related functions
;

_UsageMessage(uMsg,title="",quit=true,timeout=0)
{
  if !title
    title := _FileBaseName(A_ScriptName)
  if timeout
    MsgBox, 4160, %title%, %uMsg%,%timeout%
  else
    MsgBox, 4160, %title%, %uMsg%
  if quit
      ExitApp
}

_ErrorMessage(eMsg,title="",quit=true,timeout=0)
{
  if !title
    title := _FileBaseName(A_ScriptName)
  if timeout
    MsgBox, 4112, %title%, %eMsg%,%timeout%
  else
    MsgBox, 4112, %title%, %eMsg%
  if quit
      ExitApp
}

; returns non 0 if program is running
_IsRunning(program)
{
    Process,Exist,%program%
    return errorlevel
}

_LoWord(arg)
{
  return arg & 0xFFFF
}

_HiWord(arg)
{
  return arg >> 16
}

_LoByte(arg)
{
  return arg & 0x00FF
}

_HiByte(arg)
{
  return (arg & 0xFF00) >> 8
}

_WinVersionMajor()
{
  return _LoByte(_LoWord(DllCall("kernel32.dll\GetVersion", "UInt", -1)))
}

_WinVersionMinor()
{
  return _HiByte(_LoWord(DllCall("kernel32.dll\GetVersion", "UInt", -1)))
}

_WinVersion()
{
  dwVersion := DllCall("kernel32.dll\GetVersion", "UInt", -1)
  return _LoByte(_LoWord(dwVersion)) . "."
  . _HiByte(_LoWord(dwVersion))
}

; returns 1 if 64 bit Windows : 0 otherwise
_64BitOS()
{
  return _DirExist(A_WinDir . "\SysWow64")
}

; returns 1 if system language is US, UK, AUS, or CAN English : 0 otherwise
_IsEngLang()
{
  if (A_Language = "0409") || (A_Language = "0809") || (A_Language = "0c09") || (A_Language = "1009")
    return 1
  return 0
}

; WinAPI GetForegroundWindow - returns HWND
_GetForegroundWindow()
{
  return DllCall("GetForegroundWindow",UInt)
}

; use #IfWinActive ahk_group ExplorerGroup
; to make sure the active window is an
; Explorer window before calling this function
;
ActiveFolderPath()
{
   return PathCreateFromURL( ExplorerPath(WinExist("A")) )
}

; slightly modified version of function by jethrow
; on AHK forums.
;
ExplorerPath(_hwnd)
{
   for Item in ComObjCreate("Shell.Application").Windows
      if (Item.hwnd = _hwnd)
         return, Item.LocationURL
}

; function by SKAN on AHK forums
;
PathCreateFromURL( URL )
{
 VarSetCapacity( fPath, Sz := 2084, 0 )
 DllCall( "shlwapi\PathCreateFromUrl" ( A_IsUnicode ? "W" : "A" )
         , Str,URL, Str,fPath, UIntP,Sz, UInt,0 )
 return fPath
}

;call EmptyWorkingSet in AHK
_EmptyWorkingSet()
{
  return DllCall("psapi.dll\EmptyWorkingSet", "UInt", -1)
}

;reduce memory footprint by calling EmptyWorkingSet
_ReduceMemory()
{
  If _WinVersionMajor() > 4
  {
    return _EmptyWorkingSet()
  }
  return false
}

;return the exe that would open "file" via association
;returns "" on failure and sets ErrorLevel to system error
;or function result, non-empty exe path on success
_FindExecutable(file)
{
  ;MAX_PATH is only 256 so leave a bit more room for unicode chars
  maxpath := 2048 * 2
  exepath := ""
  VarSetCapacity(exepath,maxpath)
  result := DllCall("Shell32\FindExecutable",str,file,str,0,str,exepath,Ptr)
  if (ErrorLevel)
    ErrorLevel := A_LastError
  else if (result < 33)
    ErrorLevel := result
  return exepath
}

;function by HotKeyIt on AHK forums
FormatMessageFromSystem(ErrorCode)
{
  VarSetCapacity(Buffer, 2000)
  DllCall("FormatMessage"
          , "UInt", 0x1000      ; FORMAT_MESSAGE_FROM_SYSTEM
          , "UInt", 0
          , "UInt", ErrorCode
          , "UInt", 0x800 ;LANG_SYSTEM_DEFAULT (LANG_USER_DEFAULT=0x400)
          , "UInt", &Buffer
          , "UInt", 500
          , "UInt", 0)
  VarSetCapacity(buffer,-1)
  return Buffer
}

_GetWorkArea(ByRef left, ByRef top, ByRef right, ByRef bottom)
{
  SysGet,Mon,MonitorWorkArea
  left := MonLeft
  top := MonTop
  right := MonRight
  bottom := MonBottom
  return 1
}

_SetWorkArea(left, top, right, bottom)
{
  VarSetCapacity(work_area,16,0)
  NumPut(left,work_area,0,"Int")
  NumPut(top,work_area,4,"Int")
  NumPut(right,work_area,8,"Int")
  NumPut(bottom,work_area,12,"Int")
  return DllCall( "SystemParametersInfo", "uint", 0x2f, "uint"
    , 0, "uint", &work_area, "uint", 0 )
}

_MarginWorkArea(ByRef left, ByRef top, ByRef right, ByRef bottom, ByRef margin = 4)
{
  If _GetWorkArea(left, top, Right, Bottom)
  {
    If Margin is Integer
    {
      If (Margin < 0) or (Margin > 12)
        Margin := 4
    }
    Else
      Margin := 4
    left += Margin
    top += Margin
    Right -= Margin
    Bottom -= Margin
    return 1
  }
  return 0
}

_TaskbarGapArea(ByRef left, ByRef top, ByRef right, ByRef bottom,  percentgap = 5)
{
  _GetWorkArea(left, top, right, bottom)
  If (right - left = A_ScreenWidth) And (bottom - top = A_ScreenHeight)
  {
    return
  }
  Else If (percentgap < 1)
  {
    return
  }
  Else If (bottom < A_ScreenHeight)
  {
    bottom -= (bottom // 100 * percentgap)
  }
  Else If (top > 0)
  {
    top += (bottom - top) // 100 * percentgap
  }
  Else If (left > 0)
  {
    left += (right - left) // 100 * percentgap
  }
  Else
 {
    right -= (right - left) // 100 * percentgap
 }
}

_GlassEnabled()
{
  isEnabled = false
  If _WinVersionMajor() < 6
  {
    return false
  }
  DllCall("dwmapi.dll\DwmIsCompositionEnabled", "UInt", &isEnabled)
  return %isEnabled%
}

_EnableBlurBehind(hwnd)
{
  VarSetCapacity(struct, 16, 0)
  NumPut(1, struct, 0, "UInt") ; dwFlags=DWM_BB_Enable
  NumPut(1, struct, 4, "Int") ; fEnable=TRUE
  DllCall("dwmapi.dll\DwmEnableBlurBehindWindow", "UInt", hwnd, "UInt", &struct)
}

;returns string, wrapped in quotes when containing spaces
;updated to test if string already wrapped in double quotes
_WrapQuotes(str)
{
  wqLeftChar := ""
  wqRightChar := ""
  if (!str) || (!InStr(str," "))
    return str
  StringLeft,wqLeftChar,str,1
  StringRight,wqRightChar,str,1
  if (wqLeftChar = """") && (wqRightChar = """")
    return str
  str := """" . str . """"
  return str
}

_DirExist(dir)
{
  return InStr(FileExist(dir), "D")
}

_FilePathNoExt(path)
{
  return _FileDirWithSlash(path) . _FileBaseName(path)
}

_FileExt(path)
{
  SplitPath,path,,,ext
  return ext
}

_FileBaseName(path)
{
  if (path = "")
    return ""
  SplitPath,path,,,,basename
  return basename
}

_FileName(path)
{
  SplitPath,path,filename
  return filename
}

_FileDir(path)
{
  SplitPath,path,,fdir
  return fdir
}

_FileDirWithSlash(path)
{
  return _FileDir(path) . "\"
}

_PathIsRelative(path)
{
  return DllCall("shlwapi\PathIsRelative", "str", path)
}

; _NumberedFilePath(FilePath)
;
; FilePath should be a complete file path.
; Returns FilePath if it does not already exist.
; Otherwise returns x:\dir\file(n).ext where
; n starts at 1 and is incremented until the
; resulting FilePath does not exist.
;
_NumberedFilePath(FilePath)
{
 count := 0
 SplitPath,FilePath,,dir,ext,basename
 while FileExist(FilePath)
 {
  count += 1
  FilePath := dir "\" basename "(" count ")." ext
 }
 return FilePath
}

; AbsolutePath(Path = "", Base = "")
; Author: Compeek on AHK forums
; http://www.autohotkey.com/forum/viewtopic.php?t=43623
;
; Handles both folder and file paths for Path param.
;
; Returns 0 if Path does not exist, Absolute Path otherwise.
; For Folders it includes the trailing slash.
;

AbsolutePath(Path = "", Base = "")
{
   OriginalWorkingFolder := A_WorkingDir ; remember original working folder

   ; Set working folder to base path if specified
   If(Base) {
      SetWorkingDir, %Base%
      If(ErrorLevel) { ; if base path invalid
         SetWorkingDir, %OriginalWorkingFolder%
         Return False
      }
   }

   ; Get folder and filename if specified
   If(Path != "") {
      Attributes := FileExist(Path) ; get attributes of file/folder
      If(!Attributes) { ; if file/folder does not exist
         SetWorkingDir, %OriginalWorkingFolder%
         Return False
      }

      If(InStr(Attributes, "D")) ; if path is folder
         Folder := Path
      Else
         SplitPath, Path, Filename, Folder ; get folder and filename separately
   }

   ; Set working folder to folder if specified
   If(Folder) {
      SetWorkingDir, %Folder%
      If(ErrorLevel) { ; if path invalid
         SetWorkingDir, %OriginalWorkingFolder%
         Return False
      }
   }

   ; Get absolute path
   Path := A_WorkingDir . (SubStr(A_WorkingDir, StrLen(A_WorkingDir), 1) = "\" ? "" : "\") . (Filename ? Filename : "")
   StringReplace, Path, Path, /, \, 1

   
   SetWorkingDir, %OriginalWorkingFolder%
   
   
   Return %Path%
}

; InWorkingDir(path)
; returns true if path is contained in working directory.
; 
_InWorkingDir(path)
{
  return SubStr(_FileDir(AbsolutePath(path)),1,StrLen(A_WorkingDir)) = A_WorkingDir
}

; _PathResolve(filename,LookyHere = "")
;
; Uses PathResolve WinAPI call.
;
; Searches for filename first in LookyHere then in Path. If found
; the full path is returned. If LookyHere is left blank, then
; A_WorkingDir is used. Returns "" if filename not found.
;
_PathResolve(filename,LookyHere = "")
{
  max_path := 32768 * 2
  max_dirpath := (LokkyHere) ? StrLen(LookyHere) * 2 + 2 : StrLen(A_WorkingDir) * 2 + 2
  file_path := ""
  VarSetCapacity(fPath,max_path,0)
  VarSetCapacity(fWDir,max_dirpath,0)
  fWDir := (LookyHere) ? LookyHere : A_WorkingDir
  fPath := filename
  if DllCall("Shell32.dll\PathResolve","WStr",fPath,"WStr*",fWDir,"UInt",0)
    file_path := fPath
  fPath := 0
  fWDir := 0
  return file_path
}

; _GetRunningCopies(ExeName)
; returns the number of copies of
; executable ExeName that are currently
; running.
;
_GetRunningCopies(ExeName)
{
  nrunning := 0
  for proc in ComObjGet("winmgmts:").ExecQuery("Select * from Win32_Process")
    if (proc.Name = ExeName)
      nrunning += 1
  return nrunning
}

_KillOtherCopies()
{
    nkilled := 0
    if !A_IsCompiled
        return
    PID := DllCall("GetCurrentProcessId")
    for proc in ComObjGet("winmgmts:").ExecQuery("Select * from Win32_Process")
    if (proc.Name = A_ScriptName) && (proc.ProcessID != PID)
        {
            PPID := proc.ProcessID
            ;MsgBox, 4160, , Killing %PPID%, 2
            Process,close,%PPID%
            nkilled += 1
        }
        return nkilled
}

GetOldestCopyProcess(sName)
{
   for process in ComObjGet("winmgmts:").ExecQuery("Select Name,ProcessID,CreationDate from Win32_Process")
   {
      if (process.Name <> sName) 
         continue
         
      if !OldestPID {
         OldestPID := process.ProcessID
         OldestPIDCreationDate := process.CreationDate
      }
      else if (OldestPIDCreationDate > process.CreationDate) {
         OldestPID := process.CreationDate
         OldestPIDCreationDate := process.CreationDate
      }

   }
   return OldestPID
}

_CloseAllButOldest(ExeName)
{
   nclosed := 0
   PPID := 0
   PID := GetOldestCopyProcess(ExeName)
      
   for proc in ComObjGet("winmgmts:").ExecQuery("Select * from Win32_Process")
   if (proc.Name = ExeName) && (proc.ProcessID != PID)
   {
      PPID := proc.ProcessId
      Process,close,%PPID%
      if ErrorLevel
         nclosed += 1
   }
   return nclosed
}

; sets ErrorLevel to A_LastError
; 0 indicates success
_ShellExecute(file,params="",workdir="",verb="open",show=1)
{
  DllCall( "Shell32\ShellExecute" ( A_IsUnicode ? "W" : "A" ), "Ptr", 0, "str", verb,"str",file,"str",params,"str",workdir,int,show,int)
  ErrorLevel := A_LastError
}

_CheckForUpdate(url="http://www.favessoft.com/",ininame="FavesVersions.ini")
{
  urlfile := url . ininame
  localfile := A_ScriptDir . "\" . ininame
  param := _FileBaseName(A_ScriptFullPath)
  If (FileExist(localfile))
  {
    FileDelete,%localfile%
  }
  ;MsgBox, 4160, , localfile is %localfile%
  URLDownloadToFile,%urlfile%,%localfile%
  If ErrorLevel
  {
    MsgBox, 4112, %param%, Server access failed!
    return
  }
  IniRead,onlinever,%localfile%,Versions,%param%,%A_Space%
  IniRead,zipfile,%localfile%,Downloads,%param%,%A_Space%
  FileGetVersion,localver,%A_ScriptFullPath%
  If localver =
  {
    MsgBox, 4112, %param%, local Version Info Not Found!
    return
  }
  If onlinever =
  {
    MsgBox, 4112, %param%, Online Version Info Not Found!
    return
  }
  ;MsgBox, 4160, URLDown Online Verson, %onlinever%
  ;MsgBox, 4160, URLDown local Verson, %localver%

  If (onlinever > localver)
  {
    If zipfile =
    {
      MsgBox, 4112, %param%, Could not determine Download Filename!
      return
    }
    MsgBox, 0x1024, %param%, Current Version is %localver% : Download %onlinever% ?
    IfMsgBox Yes
    {
      urlfile := url . zipfile
      ziplocalfile := A_ScriptDir . "\" . zipfile
      URLDownloadToFile,%urlfile%,%ziplocalfile%
      If ErrorLevel
      {
        MsgBox, 4112, %param%, Download attempt failed!!
        return
      }
      Run,%ziplocalfile%
      Run,%A_ScriptDir%
      If (FileExist(localfile))
      {
        FileDelete,%localfile%
      }
      ExitApp
    }
  }
  Else
  {
    MsgBox, 4160, %param%, %localver% is the Latest Version
  }
  If (FileExist(localfile))
  {
    FileDelete,%localfile%
  }
  return
}

; func written by Lexikos
;
; Usage: WinGetProcessName("A") to get program name for active window
; e.g. chromium browser window should return chrome.exe
;
WinGetProcessName(WinTitle="", WinText="", ExcludeTitle="", ExcludeText="")
{
  WinGet pid, PID, %WinTitle%, %WinText%, %ExcludeTitle%, %ExcludeText%
  if !hproc := DllCall("OpenProcess", "uint", 0x410, "int", false, "uint", pid)
      return
  VarSetCapacity(name, 520)
  static hmod := DllCall("LoadLibrary", "str", "psapi") ; Preload psapi.dll.
  DllCall("psapi\GetModuleBaseName", "ptr", hproc, "ptr", 0, "str", name, "uint", 260)
  DllCall("CloseHandle", "ptr", hproc)
  return name
}
 
; Usage: WinGetProcessPath("A") to get program path for active window
; returns "" on error, program path on success
;
; e.g. chromium browser might return C:\Program Files\Chromium\chrome.exe
; 
WinGetProcessPath(WinTitle="", WinText="", ExcludeTitle="", ExcludeText="")
{
  WinGet pid, PID, %WinTitle%, %WinText%, %ExcludeTitle%, %ExcludeText%
  if !hproc := DllCall("OpenProcess", "uint", 0x410, "int", false, "uint", pid)
      return
  VarSetCapacity(name, 520)
  static hmod := DllCall("LoadLibrary", "str", "psapi") ; Preload psapi.dll.
  ;DllCall("psapi\GetModuleFileNameEx", "ptr", hproc, "ptr", 0, "str", name, "uint", 260)
  DllCall("psapi\GetModuleFileNameEx" ( A_IsUnicode ? "W" : "A" ), "ptr", hproc, "ptr", 0, "str", name, "uint", 260)
  DllCall("CloseHandle", "ptr", hproc)
  return name
}

; returns window handle owned by first found process named progname
;
_WindowHandleFromExe(progname)
{
  Process,Exist,%progname%
  if !ErrorLevel
    return 0
  WinGet, HWND, ID, ahk_pid %ErrorLevel%
  return HWND
}

; _DebugBox(File, LineNo, Scite)
;
; display file and line automatically instead of using MsgBox for debugging
;
; Main script should have a global Debug.  Set Debug := 0 to have this function
; return having taken no action. Set Debug to non-zero(e.g. 1) to enable Editor
; open at line number. Make sure #includes are at the bottom of the main script.
;
; Sprinkle calls to DebugBox in your code like this:
; _DebugBox(A_LineFile, A_LineNumber)
; You can also specify 3rd param Scite if SciTE4AutoHotkey is not located in the
; default install folder.
;
_DebugBox(File, LineNo, Scite = "C:\Program Files\AutoHotkey\SciTE_rc1\SciTE.exe") 
{ 
  global Debug
  if !Debug
    return
  if A_IsCompiled
    File := RegExReplace(File,"i)exe$","ahk")
  if  FileExist(Scite) And  FileExist(File)
    return _OpenEditor(File, LineNo, Scite)
  InputBox,dummy,Debug Box,%File%`n  Line  %LineNo%,,600,,,,,,%LineNo% 
}

; Open Scite4AutoHotkey to File and Line of _DebugBox() call
;
_OpenEditor(TheScript, TheLine, TheEditor)
{
  scr := _WrapQuotes(TheScript)
  LineNo := TheLine
  if A_IsCompiled
    LineNo := _CalcExecLine(TheScript,TheLine)
  Run, %TheEditor% %scr%
  WinWaitActive,%TheScript%,,6
  if ErrorLevel
    return 0
  Send,^g
  Sleep,250
  WinActivate,Go To
  Send,%LineNo%
  Sleep,100
  Send,{Enter}
  return 1
}

; If AutoHotkey_L compiled script then call this function
; to calculate the line number of _DebugBox() calls accounting
; for blank lines and comments. Remember to put Includes for
; main script at the bottom of the script so that the line
; number calculaton will be correct.
;
_CalcExecLine(TheScript, TheLine)
{
  CompiledLine := 0
  sstr := ""
  incomment := 0
  Loop,Read,%TheScript%
  { 
    sstr := Trim(A_LoopReadLine)
    if !sstr
    {
      CompiledLine := CompiledLine + 1
      Continue
    }
    StringLeft,sstr,sstr,2
    If (sstr = "/*")
    { 
        incomment := 1
        CompiledLine := CompiledLine + 1
        Continue  
    }  
    StringRight,sstr,sstr,2
    If (sstr = "*/")
    { 
        incomment := 0
        CompiledLine := CompiledLine + 1
        Continue  
    }  
    If (incomment)
    { 
        CompiledLine := CompiledLine + 1
        Continue  
    }  
    StringLeft,sstr,sstr,1
    if (sstr = ";")
    {
        CompiledLine := CompiledLine + 1
        Continue  
    }  
    CompiledLine := CompiledLine + 1
    TheLine := TheLine - 1
    if !TheLine
      return CompiledLine
  }  
  return CompiledLine
}

_ScriptVersion(script)
{
  sstr := ""
  sext := ""
  ver := ""
  pos := 0
  StringRight,sext,script,4
  if (sext = ".exe")
  {
    FileGetVersion,ver,%script%
    return ver
  }
  loop, Read, %script%
  {
    
    if (A_Index = 1) && (A_LoopReadLine != "/*")
      return ver
    if InStr(A_LoopReadLine,"File_Version=")
    {
      Stringmid,ver,A_LoopReadLine,InStr(A_LoopReadLine,"=") + 1
      return ver
    }
  }
  return ""
}

; returns true if Desktop Active and has no icon selected
;
; ( With help from Maestr0 and Melba of AHK forum )
; ( http://www.autohotkey.com/community/viewtopic.php?f=1&t=88288&sid=706a7000286344373c2a4ae654620cb3#p548578 )
;
_DesktopBlankSpot()
{
  LVM_GETSELECTEDCOUNT := 0x1000 + 50
  WinGetClass, Class, A
  if (Class != "WorkerW") and (Class != "Progman")
    return false
  handle := WinExist("A")
  handle := DllCall("GetWindow","Ptr",handle,"Uint",5,"Ptr")
  if (! handle)
    return false
  handle := DllCall("GetWindow","Ptr",handle,"Uint",5,"Ptr")
  if (! handle)
    return false
  SendMessage,%LVM_GETSELECTEDCOUNT%,0,0,,ahk_id %handle%
  return (! ErrorLevel) ; nothing selected = clicked on blank spot
}
